package main

import (
	"bytes"
	"context"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"net/url"
	"strconv"
	"strings"
	"sync"
	"time"

	"flag"

	"github.com/IBM/sarama"
	"github.com/gofiber/fiber/v2"

	gogoPbProto "github.com/gogo/protobuf/proto"
	"github.com/odysseus/go-kafka/version"
	pbUpstream "github.com/odysseus/odysseus-protocol/gen/proto/go/base/v1"
	"github.com/odysseus/service-registry/registry"
	qdb "github.com/questdb/go-questdb-client/v2"

	"github.com/odysseus/cache/cachedata"
	"github.com/odysseus/cache/model"

	"github.com/gofiber/fiber/v2/middleware/recover"
	slogfiber "github.com/samber/slog-fiber"

	"github.com/grafana/loki-client-go/loki"
	slogloki "github.com/samber/slog-loki/v3"
	// "github.com/gofiber/fiber/v2"
	// "github.com/gofiber/fiber/v2/middleware/recover"
	// slogfiber "github.com/samber/slog-fiber"
	// "log/slog"
)

var (
	// Update with your Kafka broker addresses
	kafkaBrokers = []string{"localhost:9092"}

	callbackAddr = ""

	// Update with your Kafka topic name
	topic = "test_topic"
	// Set an appropriate buffer size based on your requirements
	maxBufferSize = 100

	aigcTopic = "pbaigc"
)

// Global counter to keep track of the messages
var counter int

// Internal slice to store messages received from Kafka
var messages = make([]string, 0, maxBufferSize)

// Mutex to synchronize access to the messages slice
var mutex = sync.Mutex{}

// Channel to send messages from the /producer endpoint to the Kafka producer goroutine
var producerMessages = make(chan string)

type bytesAndHeader struct {
	Bytes      []byte
	HttpHeader map[string]string
}

var producerMessagesBytes = make(chan bytesAndHeader, 1000)

//var TaskIdInstream = make(chan string, 1000)

// Channel to send messages from the Kafka consumer to the /consumer endpoint
var consumerMessages = make(chan string)

const TaskIdAtrr = "Task-Id"
const RequestId = "X-Kong-Request-Id"

func kafkaProducerBytes() {

	// Create a new Sarama configuration for the Kafka producer.
	config := sarama.NewConfig()

	var producer sarama.AsyncProducer
	var err error

	for {
		// Create a new Kafka producer using the specified configuration and broker addresses.
		producer, err = sarama.NewAsyncProducer(kafkaBrokers, config)
		if err != nil {

			slog.LogAttrs(context.Background(), slog.LevelError, "kafkaProducerBytes", append([]slog.Attr{}, slog.String("NewAsyncProducer", err.Error()))...)

			//slog.Error("sarama.NewAsyncProducer", "err", err.Error())
			time.Sleep(time.Second * 3)
		} else {
			break
		}

	}

	// Ensure the Kafka producer is closed when the function ends (deferred execution).
	defer producer.Close()

	for message := range producerMessagesBytes {

		v := append(sarama.ByteEncoder{}, message.Bytes...)

		//sarama.ByteEncoder =
		producer.Input() <- &sarama.ProducerMessage{
			Topic: aigcTopic,
			Value: v,
		}

		// Introduce random delay between 1 to 3 seconds for message push
		//time.Sleep(time.Duration(1+rand.Intn(3)) * time.Second)
	}

}

func kafkaConsumerBytes(done chan interface{}, req, resTopic, proofTopic string) (chan pbUpstream.TaskContent, chan pbUpstream.TaskReceipt, chan pbUpstream.TaskProof) {

	reqOutStream := make(chan pbUpstream.TaskContent, 1000)
	resOutStream := make(chan pbUpstream.TaskReceipt, 1000)
	proofOutStream := make(chan pbUpstream.TaskProof, 1000)

	go func() {

		// Create a new Sarama configuration for the Kafka producer.
		config := sarama.NewConfig()

		var consumer sarama.Consumer
		var err error

		for {
			// Create a new Kafka consumer using the specified configuration and broker addresses.
			consumer, err = sarama.NewConsumer(kafkaBrokers, config)

			if err != nil {

				slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("NewConsumer", err.Error()))...)
				time.Sleep(time.Second * 3)

			} else {
				break
			}
		}

		// Ensure the Kafka consumer is closed when the function ends (deferred execution).
		defer consumer.Close()

		var reqConsumer sarama.PartitionConsumer

		for {
			// Create a partition consumer for the specified topic, partition, and starting offset.
			// The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
			reqConsumer, err = consumer.ConsumePartition(req, 0, sarama.OffsetNewest)
			if err != nil {

				slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("ConsumePartition", err.Error()))...)
				//slog.Error("consumer.ConsumePartition", "err", err.Error())

				time.Sleep(time.Second * 3)
			} else {
				break
			}
		}

		// Ensure the partition consumer is closed when the function ends (deferred execution).
		defer reqConsumer.Close()

		for {
			select {
			case message := <-reqConsumer.Messages():

				if message == nil {
					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("topic", req), slog.String("message", err.Error()))...)

					//slog.Error("kafka consumer", "topic", req, "message", "is nil")
					continue
				}

				if message.Value == nil {
					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("topic", req), slog.String("value", err.Error()))...)
					//slog.Error("kafka consumer", "topic", req, "message vaule", "is nil")
					continue
				}

				pbMsg := pbUpstream.TaskContent{}

				if err := gogoPbProto.Unmarshal(message.Value, &pbMsg); err != nil {

					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("topic", req), slog.String("pbUnmarshal", err.Error()))...)
					//slog.Error("kafka consumer", "topic", req, "gogoPbProto.Unmarshal err", err.Error())
					continue
				}

				baseAttributes := []slog.Attr{}
				baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, pbMsg.TaskId))
				slog.LogAttrs(context.Background(), slog.LevelInfo, "<- kafka consumer", append(baseAttributes, slog.String("topic", req))...)

				select {
				case <-done:
					return
				case reqOutStream <- pbMsg:
				}
			}
		}

	}()

	go func() {
		// Create a new Sarama configuration for the Kafka producer.
		config := sarama.NewConfig()

		var consumer sarama.Consumer
		var err error

		for {
			// Create a new Kafka consumer using the specified configuration and broker addresses.
			consumer, err = sarama.NewConsumer(kafkaBrokers, config)

			if err != nil {

				slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("NewConsumer", err.Error()))...)
				time.Sleep(time.Second * 3)

			} else {
				break
			}
		}

		// Ensure the Kafka consumer is closed when the function ends (deferred execution).
		defer consumer.Close()

		// Create a partition consumer for the specified topic, partition, and starting offset.
		// // The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
		// reqConsumer, err := consumer.ConsumePartition(req, 0, sarama.OffsetNewest)
		// if err != nil {
		// 	log.Fatal("Failed to start partition consumer:", err)
		// }

		// // Ensure the partition consumer is closed when the function ends (deferred execution).
		// defer reqConsumer.Close()

		// resConsumer, err := consumer.ConsumePartition(resTopic, 0, sarama.OffsetNewest)
		// if err != nil {
		// 	panic(fmt.Sprint("Failed to start partition consumer:", err))
		// }

		var resConsumer sarama.PartitionConsumer

		for {
			// Create a partition consumer for the specified topic, partition, and starting offset.
			// The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
			resConsumer, err = consumer.ConsumePartition(resTopic, 0, sarama.OffsetNewest)
			if err != nil {
				slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("ConsumePartition", err.Error()), slog.String("topic", resTopic))...)
				//slog.Error("consumer.ConsumePartition", "err", err.Error())
				time.Sleep(time.Second * 3)
			} else {
				break
			}
		}

		// Ensure the partition consumer is closed when the function ends (deferred execution).
		defer resConsumer.Close()

		// Signal that the consumer goroutine is ready
		//wg.Done()

		// Infinite loop to continuously listen for messages from the partitionConsumer.Messages() channel.

		for {
			select {

			case message := <-resConsumer.Messages():

				if message == nil {

					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("message", err.Error()), slog.String("topic", resTopic))...)
					//slog.Error("kafka consumer", "topic", resTopic, "message", "is nil")

					continue
				}

				if message.Value == nil {

					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("value", err.Error()), slog.String("topic", resTopic))...)

					//slog.Error("kafka consumer", "topic", resTopic, "message vaule", "is nil")
					continue
				}

				pbMsg := pbUpstream.TaskReceipt{}

				if err := gogoPbProto.Unmarshal(message.Value, &pbMsg); err != nil {
					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("pbUnmarshal", err.Error()), slog.String("topic", resTopic))...)
					//slog.Error("kafka consumer", "topic", resTopic, "gogoPbProto.Unmarshal err", err.Error())

					continue
				}

				baseAttributes := []slog.Attr{}
				baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, pbMsg.TaskId))
				slog.LogAttrs(context.Background(), slog.LevelInfo, "<- kafka consumer", append(baseAttributes, slog.String("topic", resTopic))...)

				//slog.Info("kafka consumer", "topic", resTopic, "pbMsg.TaskId", pbMsg.TaskId)

				select {
				case <-done:
					return
				case resOutStream <- pbMsg:
				}

			case <-done:
				return
			}
		}
	}()

	go func() {
		// Create a new Sarama configuration for the Kafka producer.
		config := sarama.NewConfig()

		var consumer sarama.Consumer
		var err error

		for {
			// Create a new Kafka consumer using the specified configuration and broker addresses.
			consumer, err = sarama.NewConsumer(kafkaBrokers, config)

			if err != nil {

				slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("NewConsumer", err.Error()))...)
				time.Sleep(time.Second * 3)

			} else {
				break
			}
		}

		// Ensure the Kafka consumer is closed when the function ends (deferred execution).
		defer consumer.Close()

		// Create a partition consumer for the specified topic, partition, and starting offset.
		// // The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
		// reqConsumer, err := consumer.ConsumePartition(req, 0, sarama.OffsetNewest)
		// if err != nil {
		// 	log.Fatal("Failed to start partition consumer:", err)
		// }

		// // Ensure the partition consumer is closed when the function ends (deferred execution).
		// defer reqConsumer.Close()

		// resConsumer, err := consumer.ConsumePartition(resTopic, 0, sarama.OffsetNewest)
		// if err != nil {
		// 	panic(fmt.Sprint("Failed to start partition consumer:", err))
		// }

		var resConsumer sarama.PartitionConsumer

		for {
			// Create a partition consumer for the specified topic, partition, and starting offset.
			// The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
			resConsumer, err = consumer.ConsumePartition(proofTopic, 0, sarama.OffsetNewest)
			if err != nil {
				slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("ConsumePartition", err.Error()), slog.String("topic", proofTopic))...)
				//slog.Error("consumer.ConsumePartition", "err", err.Error())
				time.Sleep(time.Second * 3)
			} else {
				break
			}
		}

		// Ensure the partition consumer is closed when the function ends (deferred execution).
		defer resConsumer.Close()

		// Signal that the consumer goroutine is ready
		//wg.Done()

		// Infinite loop to continuously listen for messages from the partitionConsumer.Messages() channel.

		for {
			select {

			case message := <-resConsumer.Messages():

				if message == nil {

					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("message", err.Error()), slog.String("topic", proofTopic))...)
					//slog.Error("kafka consumer", "topic", resTopic, "message", "is nil")

					continue
				}

				if message.Value == nil {

					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("value", err.Error()), slog.String("topic", proofTopic))...)

					//slog.Error("kafka consumer", "topic", resTopic, "message vaule", "is nil")
					continue
				}

				pbMsg := pbUpstream.TaskProof{}

				if err := gogoPbProto.Unmarshal(message.Value, &pbMsg); err != nil {
					slog.LogAttrs(context.Background(), slog.LevelError, "kafkaConsumerBytes", append([]slog.Attr{}, slog.String("pbUnmarshal", err.Error()), slog.String("topic", proofTopic))...)
					//slog.Error("kafka consumer", "topic", resTopic, "gogoPbProto.Unmarshal err", err.Error())

					continue
				}

				baseAttributes := []slog.Attr{}
				baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, pbMsg.TaskId))
				slog.LogAttrs(context.Background(), slog.LevelInfo, "<- kafka consumer", append(baseAttributes, slog.String("topic", proofTopic))...)

				//slog.Info("kafka consumer", "topic", resTopic, "pbMsg.TaskId", pbMsg.TaskId)

				select {
				case <-done:
					return
				case proofOutStream <- pbMsg:
				}

			case <-done:
				return
			}
		}
	}()

	return reqOutStream, resOutStream, proofOutStream
}

func batchToQuestDb(done chan interface{}, reqStream chan pbUpstream.TaskContent, resStream chan pbUpstream.TaskReceipt, proofStream chan pbUpstream.TaskProof, reqTableName string, resTableName, proofTableName string, questAddr string) {

	ctx := context.TODO()
	addrOpt := qdb.WithAddress(questAddr)

	var sender *qdb.LineSender
	var err error

	for {
		sender, err = qdb.NewLineSender(ctx, addrOpt)
		if err != nil {

			slog.LogAttrs(context.Background(), slog.LevelError, "batchToQuestDb", append([]slog.Attr{}, slog.String("NewLineSender", err.Error()))...)
			time.Sleep(time.Second * 3)

		} else {
			break
		}

	}
	// Make sure to close the sender on exit to release resources.
	defer sender.Close()

	for {
		select {
		case task := <-resStream:

			nanoseconds := int64(task.TaskTimestamp)
			seconds := nanoseconds / 1e9

			baseAttributes := []slog.Attr{}
			baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, task.TaskId))
			slog.LogAttrs(context.Background(), slog.LevelDebug, "questdb <- resStream", append(baseAttributes, slog.String("TaskUid", task.TaskUid))...)

			for {

				sender.Table(resTableName).
					Symbol("type", fmt.Sprintf("%d", task.TaskType)).
					Symbol("uid", task.TaskUid).
					Symbol("profit_acc", task.TaskProfitAccount).
					Symbol("worker_acc", task.TaskWorkerAccount).
					Symbol("result", task.TaskResult).
					StringColumn("id", task.TaskId).
					TimestampColumn("time", time.Unix(seconds, nanoseconds%1e9)).
					Int64Column("fee", task.TaskFee).
					Int64Column("workload", int64(task.TaskWorkload)).
					Int64Column("out_len", int64(task.TaskOutLen)).
					Int64Column("task_duration", int64(task.TaskDuration)).
					Int64Column("exec_duration", int64(task.TaskExecuteDuration)).
					AtNow(ctx)

				err = sender.Flush(ctx)
				if err != nil {
					slog.LogAttrs(context.Background(), slog.LevelError, "batchToQuestDb", append([]slog.Attr{}, slog.String("<-resStream", err.Error()))...)
					//slog.Error("task := <-resStream", "error", err.Error())

					time.Sleep(time.Second * 3)
				} else {
					break
				}

			}

		case task := <-reqStream:

			taskFeeAsInt, err := strconv.Atoi(task.TaskFee)
			if err != nil {
				slog.LogAttrs(context.Background(), slog.LevelError, "batchToQuestDb", append([]slog.Attr{}, slog.String("<-reqStream", err.Error()))...)
				//slog.Error("task.TaskFee string to int", "error", err)

				continue
			}

			baseAttributes := []slog.Attr{}
			baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, task.TaskId))
			slog.LogAttrs(context.Background(), slog.LevelDebug, "questdb <- reqStream", append(baseAttributes, slog.String("task.TaskUid", task.TaskUid))...)

			//slog.Debug("questdb <- reqStream", "task.TaskId", task.TaskId, "task.TaskUid", task.TaskUid)

			nanoseconds := int64(task.TaskTimestamp)
			seconds := nanoseconds / 1e9

			for {

				sender.
					Table(reqTableName).
					Symbol("type", fmt.Sprintf("%d", task.TaskType)).
					Symbol("uid", task.TaskUid).
					StringColumn("id", task.TaskId).
					TimestampColumn("time", time.Unix(seconds, nanoseconds%1e9)).
					Int64Column("fee", int64(taskFeeAsInt)).
					Int64Column("in_len", int64(task.TaskInLen)).
					AtNow(ctx)

				err = sender.Flush(ctx)
				if err != nil {

					slog.LogAttrs(context.Background(), slog.LevelError, "batchToQuestDb", append([]slog.Attr{}, slog.String("sender.Flush", err.Error()))...)
					//slog.Error("task := <-reqStream", "error", err.Error())

					time.Sleep(time.Second * 3)
				} else {
					break
				}
			}

		case proof := <-proofStream:

			//proof.TaskContainerSignature

			// TaskId                 string
			// TaskFinishTimestamp    uint64
			// TaskType               uint64
			// TaskWorkload           uint64
			// TaskReqHash            string
			// TaskRespHash           string
			// TaskManagerSignature   string
			// TaskContainerSignature string
			// TaskMinerSignature     string
			// TaskProfitAccount      string
			// TaskWorkerAccount      string

			// nanoseconds := int64(proof.TaskFinishTimestamp)
			// seconds := nanoseconds / 1e9

			for {

				sender.
					Table(proofTableName).
					Symbol("TaskType", fmt.Sprintf("%d", proof.TaskType)).
					StringColumn("TaskId", proof.TaskId).
					Int64Column("TaskFinishTimestamp", int64(proof.TaskFinishTimestamp)).
					//TimestampColumn("TaskFinishTimestamp", time.Unix(seconds, nanoseconds%1e9)).
					Int64Column("TaskWorkload", int64(proof.TaskWorkload)).
					StringColumn("TaskReqHash", proof.TaskReqHash).
					StringColumn("TaskRespHash", proof.TaskRespHash).
					StringColumn("TaskManagerSignature", proof.TaskManagerSignature).
					StringColumn("TaskContainerSignature", proof.TaskContainerSignature).
					StringColumn("TaskMinerSignature", proof.TaskMinerSignature).
					StringColumn("TaskProfitAccount", proof.TaskProfitAccount).
					StringColumn("TaskWorkerAccount", proof.TaskWorkerAccount).
					AtNow(ctx)

				err = sender.Flush(ctx)
				if err != nil {

					slog.LogAttrs(context.Background(), slog.LevelError, "batchToQuestDb", append([]slog.Attr{}, slog.String("sender.Flush", err.Error()))...)
					//slog.Error("task := <-reqStream", "error", err.Error())

					time.Sleep(time.Second * 3)
				} else {
					break
				}
			}

		case <-done:
			return
		}
	}
}

func newCache(redisAddr, redisPass, mysqlIP, dbName, user, passwd string, port int) *cachedata.CacheData {

	_cache := cachedata.NewCacheData(context.Background(), cachedata.RedisConnParam{
		Addr:     redisAddr,
		Password: redisPass,
		DbIndex:  0,
	}, model.DbConfig{
		Host:   mysqlIP,
		Port:   port,
		DbName: dbName,
		Passwd: passwd,
		User:   user,
	})
	return _cache
}

var cache *cachedata.CacheData

var createTaskTableSql string = `CREATE TABLE IF NOT EXISTS 
'tasks' ( 
	id UUID, 
	time TIMESTAMP, 
	type symbol CAPACITY 128 INDEX CAPACITY 1048576, 
	uid symbol CAPACITY 1024 INDEX CAPACITY 1048576, 
	fee INT, 
	in_len INT 
   ) timestamp (time) PARTITION BY DAY WAL;`

var createBillsTableSql string = `CREATE TABLE IF NOT EXISTS 
'bills' ( 
	id UUID, 
	time TIMESTAMP, 
	type symbol CAPACITY 128 CACHE INDEX CAPACITY 8192, 
	uid symbol CAPACITY 128 CACHE INDEX CAPACITY 8192,
	fee INT, 
	out_len INT,
	workload INT, 
	task_duration INT,
	exec_duration INT,
	profit_acc symbol CAPACITY 128 CACHE INDEX CAPACITY 8192,
	worker_acc symbol CAPACITY 128 CACHE INDEX CAPACITY 8192, 
	result symbol CAPACITY 128 CACHE INDEX CAPACITY 8192
  ) timestamp (time) PARTITION BY DAY WAL;`

func createTable(ipAddr, tableSql string) error {
	//"http://localhost:9000"
	u, err := url.Parse("http://" + ipAddr + ":9000")
	if err != nil {
		return err
	}

	u.Path += "exec"
	params := url.Values{}

	params.Add("query", tableSql)

	u.RawQuery = params.Encode()
	url := fmt.Sprintf("%v", u)

	res, err := http.Get(url)

	if err != nil {
		return err
	}

	defer res.Body.Close()

	body, err := io.ReadAll(res.Body)

	if err != nil {
		return err
	}

	_ = body
	//slog.Info("createTable", "response-body", body)

	return nil
}

var taskJsonStr = `{"id":10,"desc":"测试新增","price":1000,"complexity":1,"hardware_require":"{ disk_size: 100, memory_size: 100 }","image_id":"13221312","image_url":"dasdasdasd","cmd":"{\"image_name\":\"demianhjw/aigic:0129\",\"docker_cmd\":{\"container_port\":\"5001\"},\"api_url\":\"http://127.0.0.1:%d/aigic\"}","workload":100,"api_path":"/demianhjw/aigic/0129","image_name":"demianhjw/aigic:0129","sign_url":"dsdsdsds","username":"sdsds","password":"sdsdsd","created_time":"2024-02-02T03:13:33+08:00","updated_time":"2024-02-02T03:13:33+08:00","deleted":0}`
var withBillDb bool

func main() {

	var questAddr, kafkaBroker, callbackAddrP, listenIpPort, aigcProduceTopic, aigcTaskreceiptTopic, aigcTaskProof, logserver string

	var redisAddr, redisPassWd, mysqlAddr string
	var mysqlPort int

	var mysqlDbName, mysqlUser, mysqlPassWd string

	var logLevel int

	//var redisPassWd string

	//flag.StringVar(&questAddr, "questAddr", "192.168.1.10:9009", "questDbAddr")

	flag.StringVar(&questAddr, "questAddr", "localhost:9009", "questDbAddr")
	flag.StringVar(&kafkaBroker, "kafkaBroker", "localhost:9092", "single kafka broker")
	flag.StringVar(&redisAddr, "redisAddr", "localhost:6379", "Redis Addr")
	flag.StringVar(&redisPassWd, "redisPassWd", "", "Redis password")
	flag.StringVar(&mysqlAddr, "mysqlAddr", "localhost", "onely ip address")
	flag.IntVar(&mysqlPort, "mysqlPort", 3306, "onely port")
	flag.StringVar(&mysqlDbName, "mysqlDbName", "liuxuzhong", "mysql db name")
	flag.StringVar(&mysqlUser, "mysqlUser", "liuxuzhong", "connect to mysql user")
	flag.StringVar(&mysqlPassWd, "mysqlPassWd", "12345678", "mysql user password")

	flag.StringVar(&logserver, "logserver", "http://172.31.12.187:3100", "LOKI log server")

	flag.StringVar(&callbackAddrP, "callbackIpAddr", "localhost:6001", "ip:port")
	flag.StringVar(&listenIpPort, "listenIpPort", "0.0.0.0:6001", "api listen on ip:port")
	flag.StringVar(&aigcProduceTopic, "aigcProduceTopic", "pbaigc", "produce topic, default value is: pbaigc")
	flag.StringVar(&aigcTaskreceiptTopic, "aigcTaskreceiptTopic", "taskreceipt", "consumer topic, default value is: taskreceipt")

	flag.StringVar(&aigcTaskProof, "aigcTaskProof", "taskproof", "TaskProof topic, default value is: TaskProof")

	flag.BoolVar(&withBillDb, "withbilldb", true, "enable with bill db, or simulate")

	flag.IntVar(&logLevel, "loglevel", -4, "log level,default is debug,LevelDebug Level = -4  LevelInfo  Level = 0 LevelWarn  Level = 4 LevelError Level = 8 ")

	flag.Parse()

	detailVar.KafkaBroker = kafkaBroker
	detailVar.QuestAddr = questAddr

	detailVar.CallbackAddr = callbackAddr
	detailVar.CallbackPort = callbackAddrP
	detailVar.ListenIpPort = listenIpPort
	detailVar.AigcProduceTopic = aigcProduceTopic
	detailVar.TaskreceiptTopic = aigcTaskreceiptTopic
	detailVar.TaskProof = aigcTaskProof
	detailVar.Logserver = logserver
	detailVar.LogLevel = logLevel

	detailVar.RedisAddr = redisAddr
	detailVar.RedisPassWd = redisPassWd
	detailVar.MysqlAddr = mysqlAddr
	detailVar.MysqlDbName = mysqlDbName
	detailVar.MysqlPassWd = mysqlPassWd
	detailVar.MysqlUser = mysqlUser
	detailVar.MysqlPort = mysqlPort

	go func() {

		r := registry.NewRegistry(registry.RedisConnParam{
			Addr:     redisAddr,
			Password: redisPassWd,
			DbIndex:  0,
		}, demoService{})
		r.Start()

	}()

	levelVar := &slog.LevelVar{}
	levelVar.Set(slog.Level(logLevel))

	slog.Info("log level", "level", levelVar.String(), "log server", logserver, "version", version.VERSION)

	// slog.SetDefault(slog.New(slog.NewTextHandler(os.Stdout, &slog.HandlerOptions{
	// 	Level: levelVar,
	// 	//AddSource: true,
	// })))
	//"http://124.193.167.71:3100"

	config, _ := loki.NewDefaultConfig(logserver + "/loki/api/v1/push")
	config.TenantID = "api"
	client, _ := loki.New(config)

	//fmt.Println("Version", version.VERSION)

	slog.SetDefault(slog.New(slogloki.Option{Level: levelVar, Client: client}.NewLokiHandler()).With("api", version.VERSION))

	defer client.Stop()

	questAttributes := []slog.Attr{
		slog.String("addr", questAddr),
	}

	kafkaAttributes := []slog.Attr{
		slog.String("addr", kafkaBroker),
		slog.String("task-topic", aigcProduceTopic),
		slog.String("bill-topic", aigcTaskreceiptTopic),
		slog.String("proof-topic", aigcTaskProof),
	}

	mysqlAttributes := []slog.Attr{
		slog.String("addr", mysqlAddr),
		slog.Int("port", mysqlPort),
		slog.String("DbName", mysqlDbName),
		slog.String("User", mysqlUser),
		slog.String("PassWd", mysqlPassWd),
	}

	redisAttributes := []slog.Attr{
		slog.String("addr", redisAddr),
		slog.String("passWd", redisPassWd),
	}

	baseAttributes := []slog.Attr{}
	baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, "------------------------------"))
	baseAttributes = append(baseAttributes, slog.String("callback", callbackAddrP))
	baseAttributes = append(baseAttributes, slog.String("listenIpPort", listenIpPort))
	baseAttributes = append(baseAttributes, slog.Bool("withBillDB", withBillDb))
	baseAttributes = append(baseAttributes, slog.String("version", version.VERSION))
	baseAttributes = append(baseAttributes, slog.String("buildTime", version.BUILD_TIME))
	baseAttributes = append(baseAttributes, slog.String("gitBranch", version.GIT_BRANCH))
	baseAttributes = append(baseAttributes, slog.String("gitCommit", version.COMMIT_SHA1))

	baseAttributes = append(baseAttributes, slog.String("gitBranch", version.GIT_BRANCH))
	baseAttributes = append(baseAttributes, slog.String("loglevel", levelVar.String()))

	attributes := append(
		[]slog.Attr{
			{
				Key:   "kafka",
				Value: slog.GroupValue(kafkaAttributes...),
			},
			{
				Key:   "questDB",
				Value: slog.GroupValue(questAttributes...),
			},
			{
				Key:   "mysql",
				Value: slog.GroupValue(mysqlAttributes...),
			},
			{
				Key:   "redis",
				Value: slog.GroupValue(redisAttributes...),
			},
		},
		baseAttributes...,
	)

	// baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, reqHeaders[TaskIdAtrr][0]))
	//newAttributes := append(baseAttributes, slog.String("match_path", c.Route().Path))

	slog.LogAttrs(context.Background(), slog.LevelWarn, "start param", attributes...)

	// slog.Warn("start param", "quest", slog.GroupValue(slog.String("Addr", questAddr)))
	// slog.Warn("start param", "kafka", slog.GroupValue(slog.String("Addr", kafkaBroker)), slog.Group("topic", "aigcProduceTopic", aigcProduceTopic, "aigcConsumerTopic", aigcConsumerTopic))

	// slog.Warn("start param", "callback", slog.GroupValue(slog.String("Addr", callbackAddrP)))

	// slog.Warn("start param", "listenIpPort", slog.GroupValue(slog.String("Addr", listenIpPort)))

	// slog.Warn("start param", "redis", slog.GroupValue(slog.String("Addr", redisAddr), slog.String("PassWd", redisPassWd)))
	// slog.Warn("start param", "mysql", slog.GroupValue(slog.String("Addr", mysqlAddr),
	// 	slog.Int("port", mysqlPort), slog.String("DbName", mysqlDbName), slog.String("User", mysqlUser), slog.String("PassWd", mysqlPassWd))) //, slog.GroupValue(slog.Int("port",mysqlPort),slog.String("mysqlDbName",mysqlDbName),slog.GroupValue(slog.String("mysqlUser",mysqlUser)))))

	// slog.Warn("start param", "With-Bill-DB", withBillDb)

	//slog.Warn("start param", "!!!!!!!!!!!!!!!!!!!!**********With Bill DB**********!!!!!!!!!!!!!!!!!!!!", withBillDb)
	// slog.Warn("start param", "!!!!!!!!!!!!!!!!!!!!**********With Bill DB**********!!!!!!!!!!!!!!!!!!!!", withBillDb)
	// slog.Warn("start param", "!!!!!!!!!!!!!!!!!!!!**********With Bill DB**********!!!!!!!!!!!!!!!!!!!!", withBillDb)

	//go Registry(redisAddr, redisPassWd)

	kafkaBrokers = []string{kafkaBroker}
	callbackAddr = callbackAddrP

	idx := strings.Index(questAddr, ":")

	if idx == -1 {
		panic("quest addr parameter,please use the format: 1.1.1.1:9009")
	}

	createTable(questAddr[:idx], createTaskTableSql)
	createTable(questAddr[:idx], createBillsTableSql)

	cache = newCache(redisAddr, redisPassWd, mysqlAddr, mysqlDbName, mysqlUser, mysqlPassWd, mysqlPort)

	// Create a WaitGroup to synchronize goroutines.
	wg := &sync.WaitGroup{}
	// Add 1 to the WaitGroup, indicating one goroutine to wait for.
	wg.Add(1)

	// Launch the Kafka producer goroutine in the background.
	go kafkaProducerBytes()

	done := make(chan interface{})

	reqToQuestDb, resToQuestDb, proofToQuestDb := kafkaConsumerBytes(done, aigcProduceTopic, aigcTaskreceiptTopic, aigcTaskProof)

	go batchToQuestDb(done, reqToQuestDb, resToQuestDb, proofToQuestDb, "tasks", "bills", "proof", questAddr)
	go recordUUID()

	// Create a new instance of the Fiber web framework.
	app := fiber.New()

	// cfg := slogfiber.Config{
	// 	DefaultLevel:     slog.LevelInfo,
	// 	ClientErrorLevel: slog.LevelWarn,
	// 	ServerErrorLevel: slog.LevelError,

	// 	WithUserAgent:      false,
	// 	WithRequestID:      false,
	// 	WithRequestBody:    false,
	// 	WithRequestHeader:  true,
	// 	WithResponseBody:   false,
	// 	WithResponseHeader: false,
	// 	WithSpanID:         false,
	// 	WithTraceID:        false,

	// 	//Filters: []Filter{},
	// }
	//_ = cfg

	//app := fiber.New()
	//app.Use(slogfiber.NewWithConfig(slog.Default(), cfg))

	//app.Use(slogfiber.New(slog.Default()))
	//app.Use(recover.New())

	app.Use(slogfiber.New(slog.Default()))
	app.Use(recover.New())

	apiGroup := app.Group("/api")
	jwtGroup := app.Group("/jwt")

	apiGroupV1 := apiGroup.Group("/v1")
	apiGroupV1.Post("/*", ApiOrJWT)

	jwtGroupV1 := jwtGroup.Group("/v1")
	//jwtGroupV1.Post("/*", slogfiber.New(slog.Default()), ApiOrJWT)

	jwtGroupV1.Post("/*", ApiOrJWT)

	//curl -X GET http://127.0.0.1:4000/api/jobs?id=e45b5ebc-c71e-4ab8-b10f-d1202e7fb16e

	//slogfiber.NewWithConfig(slog.Default(), cfg)

	queryGroup := app.Group("/query")
	queryGroupV1 := queryGroup.Group("/v1")
	queryGroupV1.Get("/:taskId", func(c *fiber.Ctx) error {

		taskId := c.Params("taskId")

		slogfiber.AddCustomAttributes(c, slog.String(TaskIdAtrr, taskId))

		queryBaseAttributes := []slog.Attr{}
		queryBaseAttributes = append(queryBaseAttributes, slog.String(TaskIdAtrr, taskId))

		reqHeaders := c.GetReqHeaders()

		// if taskHeaders, ok := reqHeaders[TaskIdAtrr]; ok {
		// 	if taskHeaders == nil || len(taskHeaders) == 0 {

		// 		return ApiErrorF(c, fmt.Sprintf(fmt.Sprintf("Please provide http header %s  value, right now the value is %v.", TaskIdAtrr, taskHeaders)), queryBaseAttributes)
		// 	}
		// } else {

		// 	return ApiErrorF(c, fmt.Sprintf("Please provide http header %s .", TaskIdAtrr), queryBaseAttributes)
		// }

		if RequestIdHeaders, ok := reqHeaders[RequestId]; ok {
			if RequestIdHeaders == nil || len(RequestIdHeaders) == 0 {
				return ApiErrorF(c, fmt.Sprintf(fmt.Sprintf("Please provide http header %s  value, right now the value is %v.", RequestId, RequestIdHeaders)), queryBaseAttributes)
			}
		} else {
			return ApiErrorF(c, fmt.Sprintf("Please provide http header %s .", RequestId), queryBaseAttributes)
		}

		slogfiber.AddCustomAttributes(c, slog.String(RequestId, reqHeaders[RequestId][0]))

		// baseAttributes := []slog.Attr{}
		// baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, taskId))

		newAttributes := append(queryBaseAttributes, slog.String("c.Path()", c.Path()), slog.String("c.Route().Path", c.Route().Path))
		slog.LogAttrs(c.UserContext(), slog.LevelInfo, "http api query task", newAttributes...)

		if len(taskId) == 0 {
			return ApiErrorF(c, fmt.Sprintf("%s must provide task id param for route %s ; e.g. http://127.0.0.1/query/v1/e45b5ebc-c71e-4ab8-b10f-d1202e7fb16e ", c.Path(), c.Route().Path), newAttributes)
		}

		if res, ok := getAsyncRes(taskId); ok {

			newAttributes := append(queryBaseAttributes, slog.Int("resAsPb.TaskResultCode", int(res.TaskResultCode)))
			newAttributes = append(newAttributes, slog.String("TaskResultHeader", string(res.TaskResultHeader)))

			if isBase64(res.TaskResultBody) {
				newAttributes = append(newAttributes, slog.Int("TaskResultBody as Base64", len(res.TaskResultBody)))
			} else {
				newAttributes = append(newAttributes, slog.String("TaskResultBody is not Base64", string(res.TaskResultBody)))
			}

			slog.LogAttrs(c.UserContext(), slog.LevelInfo, "getAsyncRes", newAttributes...)

			return syncOrAsyncReturn(c, res, taskId, baseAttributes)
		}

		return ApiErrorF(c, fmt.Sprintf("can not find out the task id %s in result cache.", taskId), attributes)

	})

	cfg := slogfiber.Config{
		WithUserAgent:      true,
		WithRequestID:      true,
		WithRequestBody:    true,
		WithRequestHeader:  true,
		WithResponseBody:   true,
		WithResponseHeader: true,
		WithSpanID:         true,
		WithTraceID:        true,
		//DefaultLevel:       levelVar.Level(),
		DefaultLevel:     slog.LevelInfo,
		ClientErrorLevel: slog.LevelWarn,
		ServerErrorLevel: slog.LevelError,
	}

	callbackGroup := app.Group("/callback")
	callbackGroupV1 := callbackGroup.Group("/v1")
	callbackGroupV1.Post("/", slogfiber.NewWithConfig(slog.Default(), cfg), func(c *fiber.Ctx) error {

		slog.LogAttrs(c.UserContext(), slog.LevelInfo, "callback", slog.String("path", c.Route().Path))

		body := c.Body()

		var resbody pbUpstream.TaskResponse

		if err := gogoPbProto.Unmarshal(body, &resbody); err != nil {
			return ApiErrorF(c, fmt.Sprintf("callback Unmarshal error %v", err.Error()), attributes)
		}

		slogfiber.AddCustomAttributes(c, slog.String(TaskIdAtrr, resbody.TaskId))

		baseAttributes := []slog.Attr{}
		baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, resbody.TaskId))

		slog.LogAttrs(c.UserContext(), slog.LevelDebug, "callback", append(baseAttributes, slog.Bool("TaskIsSucceed", resbody.TaskIsSucceed), slog.String(TaskIdAtrr, resbody.TaskId), slog.String("TaskUid", resbody.TaskUid))...)

		callbackRes(resbody)

		return c.SendStatus(200)
	})

	task := model.TaskType{}

	if err := json.Unmarshal([]byte(taskJsonStr), &task); err != nil {
		panic(fmt.Sprintf("json.Unmarshal error, %s", err.Error()))
	}

	replanceQueryTask = &task

	if err := app.Listen(listenIpPort); err != nil {
		slog.LogAttrs(context.Background(), slog.LevelError, "fiber listen", append([]slog.Attr{}, slog.String("err", err.Error()))...)
	}
}

var replanceQueryTask *model.TaskType

func ApiOrJWT(c *fiber.Ctx) error {

	// cfg := slogfiber.Config{
	// 	WithUserAgent:      true,
	// 	WithRequestID:      true,
	// 	WithRequestBody:    true,
	// 	WithRequestHeader:  true,
	// 	WithResponseBody:   false,
	// 	WithResponseHeader: true,
	// 	WithSpanID:         true,
	// 	WithTraceID:        true,
	// }

	//slogfiber.NewWithConfig(slog.Default(), cfg)

	reqHeaders := c.GetReqHeaders()

	// for k, v := range reqHeaders {
	// 	fmt.Println("k", k, "v", v)
	// }

	if taskHeaders, ok := reqHeaders[TaskIdAtrr]; ok {
		if taskHeaders == nil || len(taskHeaders) == 0 {

			baseAttributes := []slog.Attr{}
			baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, "can not get "+TaskIdAtrr))

			return ApiErrorF(c, fmt.Sprintf(fmt.Sprintf("Please provide http header %s  value, right now the value is %v.", TaskIdAtrr, taskHeaders)), baseAttributes)
		}
	} else {
		baseAttributes := []slog.Attr{}
		baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, "can not get "+TaskIdAtrr))
		return ApiErrorF(c, fmt.Sprintf("Please provide http header %s .", TaskIdAtrr), baseAttributes)
	}

	baseAttributes := []slog.Attr{}
	baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, reqHeaders[TaskIdAtrr][0]))
	slogfiber.AddCustomAttributes(c, slog.String(TaskIdAtrr, reqHeaders[TaskIdAtrr][0]))

	if RequestIdHeaders, ok := reqHeaders[RequestId]; ok {
		if RequestIdHeaders == nil || len(RequestIdHeaders) == 0 {
			return ApiErrorF(c, fmt.Sprintf("Please provide http header %s  value, right now the value is %v.", RequestId, RequestIdHeaders), nil)
		}
	} else {
		return ApiErrorF(c, fmt.Sprintf("Please provide http header %s .", RequestId), baseAttributes)
	}

	slogfiber.AddCustomAttributes(c, slog.String(RequestId, reqHeaders[RequestId][0]))

	//slogfiber.AddCustomAttributes(c, slog.String(TaskIdAtrr, reqHeaders[TaskIdAtrr][0]))

	baseAttributes = append(baseAttributes, slog.String(RequestId, reqHeaders[RequestId][0]))
	newAttributes := append(baseAttributes, slog.String("match_path", c.Route().Path))

	slog.LogAttrs(c.UserContext(), slog.LevelInfo, "new Api or JWT reuqest", newAttributes...)

	routePath := c.Route().Path
	routePathWithoutStar := strings.TrimSuffix(routePath, "/*")

	path := c.Path()
	pathInDB := strings.TrimPrefix(path, routePathWithoutStar)

	uid := reqHeaders["X-Consumer-Custom-Id"]

	if uid == nil {
		return ApiErrorF(c, fmt.Sprintf("uid can not be nil"), baseAttributes)
	}

	if len(uid) == 0 {
		return ApiErrorF(c, fmt.Sprintf("len(uid) can not be 0"), baseAttributes)
	}

	uidAsInt, err := strconv.Atoi(uid[0])

	queryDbAttributes := []slog.Attr{
		slog.String("path", pathInDB),
		slog.Int("uid", uidAsInt),
	}

	attributes := append(
		[]slog.Attr{
			{
				Key:   "param",
				Value: slog.GroupValue(queryDbAttributes...),
			},
		},
		baseAttributes...,
	)

	slog.LogAttrs(c.UserContext(), slog.LevelDebug, "query db", attributes...)

	var task *model.TaskType

	if withBillDb {
		var err error
		task, err = cache.Query(pathInDB, int64(uidAsInt))

		if err != nil {
			return ApiErrorF(c, fmt.Sprintf("cache.Query %v", err.Error()), baseAttributes)
		}
	} else {
		task = replanceQueryTask
	}

	//task.ResultFileExpires

	//json:"task_result_body,omitempty"

	reqHeaders["ResultFileExpiresDB"] = []string{fmt.Sprintf("%d", task.ResultFileExpires)}

	reqRelayAsJson, err := json.Marshal(struct {
		Headers map[string][]string `json:"headers"`
		Queries map[string]string   `json:"queries"`
		Body    []byte              `json:"body"`
	}{
		Headers: reqHeaders,
		Queries: c.Queries(),
		Body:    c.Body(),
	})

	// c.Protocol()
	// map[string][]string
	// map[string]string

	pbMsg := pbUpstream.TaskContent{
		TaskId:          reqHeaders["Task-Id"][0],
		TaskType:        uint64(task.ID),
		TaskKind:        pbUpstream.TaskKind(task.Kind),
		TaskCmd:         task.Cmd,
		TaskParam:       reqRelayAsJson, //[]byte(reqHeaders["Task-Id"][0]),
		TaskTimestamp:   uint64(time.Now().UnixNano()),
		TaskCallback:    "http://" + callbackAddr + "/callback/v1", //"http://192.168.1.10:6001/callback/v1",
		TaskUid:         reqHeaders["X-Consumer-Custom-Id"][0],
		TaskFee:         fmt.Sprintf("%d", task.Price),
		TaskInLen:       int32(len(c.Body())),
		TaskWorkload:    task.Workload,
		ContainerPubkey: []byte(task.PublicKey),
	}

	msgToKafkaAttributes := []slog.Attr{
		slog.String("TaskId", pbMsg.TaskId),
		slog.Uint64("TaskType", pbMsg.TaskType),
		slog.Int("TaskKind", int(pbMsg.TaskKind)),
		slog.String("TaskCmd", pbMsg.TaskCmd),
		slog.String("TaskParam", string(pbMsg.TaskParam)),
		slog.String("TaskTimestamp", string(pbMsg.TaskTimestamp)),
		slog.String("TaskCallback", pbMsg.TaskCallback),
		slog.String("TaskUid", pbMsg.TaskUid),
		slog.String("TaskFee", pbMsg.TaskFee),
		slog.Int("TaskInLen", int(pbMsg.TaskInLen)),
		slog.Int64("TaskWorkload", pbMsg.TaskWorkload),
		slog.Int("ContainerPubkey Len", len(pbMsg.ContainerPubkey)),
	}

	kafkaattributes := append(
		[]slog.Attr{
			{
				Key:   "pb msg",
				Value: slog.GroupValue(msgToKafkaAttributes...),
			},
		},
		baseAttributes...,
	)

	slog.LogAttrs(c.UserContext(), slog.LevelDebug, "-> kafka producer", kafkaattributes...)

	// slog.Debug("msg to kafka",
	// 	"TaskId", pbMsg.TaskId,
	// 	"TaskType", pbMsg.TaskType,
	// 	"TaskKind", pbMsg.TaskKind,
	// 	"TaskCmd", pbMsg.TaskCmd,
	// 	"TaskParam", pbMsg.TaskParam,
	// 	"TaskTimestamp", pbMsg.TaskTimestamp,
	// 	"TaskCallback", pbMsg.TaskCallback,
	// 	"TaskUid", pbMsg.TaskUid,
	// 	"TaskFee", pbMsg.TaskFee,
	// 	"TaskInLen", pbMsg.TaskInLen,
	// 	"TaskWorkload", pbMsg.TaskWorkload,
	// 	"ContainerPubkey", pbMsg.ContainerPubkey)

	pbBytes, err := gogoPbProto.Marshal(&pbMsg)

	if err != nil {
		slog.LogAttrs(context.Background(), slog.LevelError, "ApiOrJWT", append([]slog.Attr{}, slog.String("pbMarshal", err.Error()))...)
		return ApiErrorF(c, fmt.Sprintf("pb error: %v", err.Error()), baseAttributes)
	}

	producerMessagesBytes <- bytesAndHeader{
		Bytes: pbBytes,
	}

	asyncMode := false

	// for k, v := range reqHeaders {
	// 	fmt.Println("k-------", k, "v", v)
	// }

	if headerSync, ok := reqHeaders["Prefer"]; ok {
		for _, syncAsString := range headerSync {
			if syncAsString == "respond-async" {
				asyncMode = true
				break
			} else {

				asyncMode = false
				break
			}

		}
	}

	//slog.Info("client", "taskid", pbMsg.TaskId, "asyncMode", asyncMode)

	asyncModeAttributes := append(baseAttributes, slog.Bool("async", asyncMode))

	slog.LogAttrs(c.UserContext(), slog.LevelInfo, "client request mode", asyncModeAttributes...)

	if asyncMode {
		//time.Sleep(10 * time.Second)
		//m := make([]byte, 1024*1024+1024*512)
		//return c.Send(m)

		asyncReq(pbMsg.TaskId)

		res := SyncResponse{
			TaskId: pbMsg.TaskId,
		}

		resAsJson, err := json.Marshal(res)

		if err != nil {
			newAttributes := append(baseAttributes, slog.String("err", err.Error()))
			slog.LogAttrs(c.UserContext(), slog.LevelError, "json.Marshal(res)", newAttributes...)
		}

		if err := c.Send(resAsJson); err != nil {
			newAttributes := append(baseAttributes, slog.String("err", err.Error()))
			slog.LogAttrs(c.UserContext(), slog.LevelError, "c.JSON(pbMsg.TaskId);", newAttributes...)
			return err
		}

		return nil

	} else {
		return syncModeF(c, pbMsg.TaskId)
	}
}

type SyncResponse struct {
	TaskId string `json:"task_id"`
}

func isBase64(s []byte) bool {
	_, err := base64.StdEncoding.DecodeString(string(s))
	return err == nil
}

func syncModeF(c *fiber.Ctx, taskid string) error {

	wait := syncReq(taskid)
	resAsPb := <-wait

	baseAttributes := []slog.Attr{}
	baseAttributes = append(baseAttributes, slog.String(TaskIdAtrr, taskid))
	// newAttributes := append(baseAttributes, slog.Int("resAsPb.TaskResultCode", int(resAsPb.TaskResultCode)), slog.String("TaskResultHeader", string(resAsPb.TaskResultHeader)), slog.String("TaskResultBody", string(resAsPb.TaskResultBody)))

	newAttributes := append(baseAttributes, slog.Int("resAsPb.TaskResultCode", int(resAsPb.TaskResultCode)))
	newAttributes = append(newAttributes, slog.String("TaskResultHeader", string(resAsPb.TaskResultHeader)))

	if isBase64(resAsPb.TaskResultBody) {
		newAttributes = append(newAttributes, slog.Int("TaskResultBody as Base64", len(resAsPb.TaskResultBody)))
	} else {
		newAttributes = append(newAttributes, slog.String("TaskResultBody", string(resAsPb.TaskResultBody)))
	}

	slog.LogAttrs(c.UserContext(), slog.LevelInfo, "syncModeF", newAttributes...)

	return syncOrAsyncReturn(c, resAsPb, taskid, baseAttributes)
}

func syncOrAsyncReturn(c *fiber.Ctx, resAsPb pbUpstream.TaskResponse, reqTaskId string, baseAttributes []slog.Attr) error {

	// 	303 (See Other) responses always lead to the use of a GET method.
	// 307 (Temporary Redirect) and 308 (Permanent Redirect) don't change the method used in the original request.
	// 301 (Moved Permanently) and 302 (Found) don't change the method most of the time, though older user-agents may (so you basically don't know).

	redirectCode := false
	if resAsPb.TaskResultCode == 301 || resAsPb.TaskResultCode == 302 || resAsPb.TaskResultCode == 303 || resAsPb.TaskResultCode == 307 || resAsPb.TaskResultCode == 308 {

		redirectCode = true
	}

	if resAsPb.TaskResultHeader != nil {
		if len(resAsPb.TaskResultHeader) != 0 {

			headers := make(map[string][]string)

			if err := json.Unmarshal(resAsPb.TaskResultHeader, &headers); err != nil {
				slog.LogAttrs(context.Background(), slog.LevelError, "syncOrAsyncReturn", append([]slog.Attr{}, slog.String("reqTaskId", reqTaskId), slog.String(TaskIdAtrr, resAsPb.TaskId), slog.String("json.Unmarshal", err.Error()))...)
				return ApiErrorF(c, fmt.Sprintf("json.Unmarshal(resAsPb.TaskResultHeader error: %v", err.Error()), baseAttributes)
			}

			for k, vs := range headers {
				for _, v := range vs {

					if k == "Content-Encoding" {

						newAttributes := append(baseAttributes,
							slog.String(k, v),
						)

						slog.LogAttrs(c.UserContext(), slog.LevelInfo, "http Content-Encoding header in body", newAttributes...)

						c.Response().Header.SetContentEncoding(v)
					}

					if k == "Content-Type" {
						newAttributes := append(baseAttributes,
							slog.String(k, v),
						)

						slog.LogAttrs(c.UserContext(), slog.LevelInfo, "http Content-Type header in body", newAttributes...)

						c.Response().Header.SetContentType(v)
					}

					if redirectCode {
						if k == "Location" {

							newAttributes := append(baseAttributes,
								slog.String(k, v),
								slog.Int("resAsPb.TaskResultCode", int(resAsPb.TaskResultCode)),
							)

							slog.LogAttrs(c.UserContext(), slog.LevelInfo, "http location header in body", newAttributes...)

							c.Redirect(v, int(resAsPb.TaskResultCode))

							//c.Response().Header.SetL
							//c.Response().Header.Set
							//c.Response().Header.SetContentType(v)
							//c.Set("Location", v)
						}
					}
				}
			}
		}
	}

	// if redirectCode {
	// 	return redirectReturn(c, resAsPb)
	// }

	return Return(c, resAsPb, baseAttributes)

}

func Return(c *fiber.Ctx, resAsPb pbUpstream.TaskResponse, baseAttributes []slog.Attr) error {

	resAsJson := ResponseJson{
		Task: ResponseTask{
			TaskId:    resAsPb.TaskId,
			TaskUid:   resAsPb.TaskUid,
			TaskFee:   resAsPb.TaskFee,
			IsSuccess: resAsPb.TaskIsSucceed,
			TaskError: resAsPb.TaskError,
			ExecCode:  fmt.Sprintf("%d", resAsPb.TaskResultCode), //resAsPb.TaskResultCode,
			//TaskExecuteError
			ExecError: resAsPb.TaskExecuteError,
		},
		Output: resAsPb.GetTaskResultBody(),
	}

	resbuffer := &bytes.Buffer{}
	encoder := json.NewEncoder(resbuffer)
	encoder.SetEscapeHTML(false)

	//fileAsJsonEscapeHTML,err  := EncodeJsonEscapeHTML(fileAsJsonBytes)

	if err := encoder.Encode(resAsJson); err != nil {
		newAttributes := append(baseAttributes, slog.String("err", err.Error()))
		slog.LogAttrs(c.UserContext(), slog.LevelError, "encoder.Encode(resAsJson)", newAttributes...)
		return err
	}

	newAttributes := append(baseAttributes,
		slog.String("TaskId", resAsJson.Task.TaskId),
		slog.String("TaskUid", resAsJson.Task.TaskUid),
		slog.String("TaskFee", resAsJson.Task.TaskFee),
		slog.Bool("IsSuccess", resAsJson.Task.IsSuccess),
		slog.String("TaskError", resAsJson.Task.TaskError),
		slog.String("ExecCode", resAsJson.Task.ExecCode),
		slog.String("ExecError", resAsJson.Task.ExecError),
		//slog.String("Output", string(resAsJson.Output)),
	)

	if isBase64(resAsPb.TaskResultBody) {
		newAttributes = append(newAttributes, slog.Int("TaskResultBody as Base64", len(resAsPb.TaskResultBody)))
	} else {
		newAttributes = append(newAttributes, slog.String("TaskResultBody", string(resAsPb.TaskResultBody)))
	}

	slog.LogAttrs(c.UserContext(), slog.LevelInfo, "http return", newAttributes...)

	if err := c.Status(int(resAsPb.TaskResultCode)).JSON(resAsJson); err != nil {
		newAttributes := append(baseAttributes, slog.String("err", err.Error()))
		slog.LogAttrs(c.UserContext(), slog.LevelError, "c.Status(int(resAsPb.TaskResultCode)).JSON(resAsJson)", newAttributes...)
		return err
	}

	return nil

	//return c.JSON(resAsJson)
}

type ResponseJson struct {
	Task   ResponseTask    `json:"task"`
	Output json.RawMessage `json:"output"`
}

type ResponseTask struct {
	TaskId string `json:"task_id"`
	//TaskResult []byte `json:"task_result"`
	TaskUid   string `json:"task_uid"`
	TaskFee   string `json:"task_fee"`
	IsSuccess bool   `json:"is_success"`
	TaskError string `json:"task_error"`
	ExecCode  string `json:"exec_code"`
	//ExecCode  int32  `json:"exec_code"`
	//  "exec_code":"",
	ExecError string `json:"exec_error"`
	//Api       ApiError `json:"api"`
	ApiError string `json:"api_error"`
}

type ApiError struct {
	Code int    `json:"code"`
	Msg  string `json:"msg"`
}

func ApiErrorF(c *fiber.Ctx, msg string, baseAttributes []slog.Attr) error {

	resAsJson := ResponseJson{
		Task: ResponseTask{
			IsSuccess: false,
			ApiError:  msg,
		},
	}

	resbuffer := &bytes.Buffer{}
	encoder := json.NewEncoder(resbuffer)
	encoder.SetEscapeHTML(false)

	//fileAsJsonEscapeHTML,err  := EncodeJsonEscapeHTML(fileAsJsonBytes)

	if err := encoder.Encode(resAsJson); err != nil {
		newAttributes := append(baseAttributes, slog.String("err", err.Error()))
		slog.LogAttrs(c.UserContext(), slog.LevelError, "encoder.Encode(resAsJson)", newAttributes...)
		return err
	}

	newAttributes := append(baseAttributes,
		slog.String("TaskId", resAsJson.Task.TaskId),
		slog.String("TaskUid", resAsJson.Task.TaskUid),
		slog.String("TaskFee", resAsJson.Task.TaskFee),
		slog.Bool("IsSuccess", resAsJson.Task.IsSuccess),
		slog.String("TaskError", resAsJson.Task.TaskError),
		slog.String("ExecCode", resAsJson.Task.ExecCode),
		slog.String("ExecError", resAsJson.Task.ExecError),
		slog.String("ApiError", resAsJson.Task.ApiError))

	slog.LogAttrs(c.UserContext(), slog.LevelInfo, "http return api error", newAttributes...)

	if err := c.Status(fiber.StatusOK).JSON(resAsJson); err != nil {
		newAttributes := append(baseAttributes, slog.String("err", err.Error()))
		slog.LogAttrs(c.UserContext(), slog.LevelError, "c.Status(fiber.StatusOK).JSON(resAsJson)", newAttributes...)
		return err
	}

	return nil
}
